{% extends "admin/delete_confirmation.html" %}
{% load i18n %}

{% block content %}
<p>{% blocktrans with object as escaped_object %}Deletion of the {{ language_name }} translation of {{ object_name }} '{{ escaped_object }}' is not allowed, because it is the last available translation of this instance.{% endblocktrans %}</p>
{% endblock %}
